@@ -164,6 +164,8 @@ gail_cell_object_init (GailCell *cell)
   cell->index = 0;
   atk_state_set_add_state (cell->state_set, ATK_STATE_TRANSIENT);
   atk_state_set_add_state (cell->state_set, ATK_STATE_ENABLED);
+  atk_state_set_add_state (cell->state_set, ATK_STATE_SENSITIVE);
+  atk_state_set_add_state (cell->state_set, ATK_STATE_SELECTABLE);
   cell->refresh_index = NULL;
 }
 
@@ -171,6 +173,11 @@ static void
 gail_cell_object_finalize (GObject *obj)
 {
   GailCell *cell = GAIL_CELL (obj);
+  AtkRelationSet *relation_set;
+  AtkRelation *relation;
+  GPtrArray *target;
+  gpointer target_object;
+  gint i;
 
   if (cell->state_set)
     g_object_unref (cell->state_set);
@@ -180,6 +187,25 @@ gail_cell_object_finalize (GObject *obj)
       g_source_remove (cell->action_idle_handler);
       cell->action_idle_handler = 0;
     }
+  relation_set = atk_object_ref_relation_set (ATK_OBJECT (obj));
+  if (ATK_IS_RELATION_SET (relation_set))
+    {
+      relation = atk_relation_set_get_relation_by_type (relation_set, 
+                                                        ATK_RELATION_NODE_CHILD_OF);
+      if (relation)
+        {
+          target = atk_relation_get_target (relation);
+          for (i = 0; i < target->len; i++)
+            {
+              target_object = g_ptr_array_index (target, i);
+              if (GAIL_IS_CELL (target_object))
+                {
+                  g_object_unref (target_object);
+                }
+            }
+        }
+      g_object_unref (relation_set);
+    }
   G_OBJECT_CLASS (parent_class)->finalize (obj);
 }
 
@@ -242,6 +268,8 @@ gail_cell_remove_state (GailCell     *cell,
       gboolean rc;
       AtkObject *parent;
 
+      parent = atk_object_get_parent (ATK_OBJECT (cell));
+
       rc = atk_state_set_remove_state (cell->state_set, state_type);
       /*
        * The signal should only be generated if the value changed,
@@ -262,7 +290,6 @@ gail_cell_remove_state (GailCell     *cell,
        * change to it also 
        */
 
-      parent = atk_object_get_parent (ATK_OBJECT (cell));
       if (GAIL_IS_CONTAINER_CELL (parent))
         gail_cell_remove_state (GAIL_CELL (parent), state_type, emit_signal);
       return rc;
@@ -490,10 +517,14 @@ idle_do_action (gpointer data)
 {
   GailCell *cell;
 
+  GDK_THREADS_ENTER ();
+
   cell = GAIL_CELL (data);
   cell->action_idle_handler = 0;
   cell->action_func (cell);
 
+  GDK_THREADS_LEAVE ();
+
   return FALSE;
 }
 
